@@ -1,4 +1,3 @@
-#!/usr/bin/python3
 import os
 import json
 import requests
@@ -44,7 +43,7 @@ from deezspot.libutils.progress_reporter import (
     report_album_initializing, report_album_done, report_playlist_initializing, report_playlist_done
 )
 from deezspot.libutils.taggers import (
-    enhance_metadata_with_image, add_deezer_enhanced_metadata, process_and_tag_track,
+    enhance_metadata_with_image, add_deezer_enhanced_metadata, add_spotify_enhanced_metadata, process_and_tag_track,
     save_cover_image_for_track
 )
 from mutagen.flac import FLAC
@@ -187,15 +186,15 @@ class Download_JOB:
                 # Post-process each returned media in the chunk
                 for idx in range(len(chunk_medias)):
                     if "errors" in chunk_medias[idx]:
-                        c_media_json = cls.__get_url(non_episode_tracks[len(non_episode_medias) + idx], quality_download)
-                        chunk_medias[idx] = c_media_json
+                        # Do not fallback to legacy URL; mark as unavailable for this quality
+                        chunk_medias[idx] = {"media": []}
                     else:
                         if not chunk_medias[idx]['media']:
-                            c_media_json = cls.__get_url(non_episode_tracks[len(non_episode_medias) + idx], quality_download)
-                            chunk_medias[idx] = c_media_json
+                            # Do not fallback to legacy URL; mark as unavailable
+                            chunk_medias[idx] = {"media": []}
                         elif len(chunk_medias[idx]['media'][0]['sources']) == 1:
-                            c_media_json = cls.__get_url(non_episode_tracks[len(non_episode_medias) + idx], quality_download)
-                            chunk_medias[idx] = c_media_json
+                            # Keep single-source media as-is; do not fallback
+                            pass
                 non_episode_medias.extend(chunk_medias)
             except NoRightOnMedia:
                 for c_track in tokens_chunk:
@@ -264,10 +263,45 @@ class EASY_DW:
         else:
             # Get the track object from preferences
             self.__track_obj: trackCbObject = preferences.song_metadata
+            # If spotify metadata flag is set and a spotify track object is provided, prefer it for tagging
+            if getattr(preferences, 'spotify_metadata', False) and getattr(preferences, 'spotify_track_obj', None):
+                self.__track_obj = preferences.spotify_track_obj
             
             # Convert it to the dictionary format needed for legacy functions
-            self.__song_metadata = self._track_object_to_dict(self.__track_obj)
+            artist_separator = getattr(preferences, 'artist_separator', '; ')
+            # Auto-select source type based on preference
+            use_spotify = getattr(preferences, 'spotify_metadata', False)
+            source_type = 'spotify' if use_spotify else 'deezer'
+            self.__song_metadata_dict = track_object_to_dict(self.__track_obj, source_type=source_type, artist_separator=artist_separator)
+            # Maintain legacy attribute expected elsewhere
+            self.__song_metadata = self.__song_metadata_dict
             self.__download_type = "track"
+            # Backfill missing album fields when using Spotify minimal track objects
+            try:
+                if use_spotify and 'album' not in self.__song_metadata_dict and getattr(preferences, 'spotify_album_obj', None):
+                    spo_album = preferences.spotify_album_obj
+                    # Basic album fields
+                    self.__song_metadata_dict['album'] = getattr(spo_album, 'title', '')
+                    if getattr(spo_album, 'artists', None):
+                        self.__song_metadata_dict['ar_album'] = artist_separator.join([getattr(a, 'name', '') for a in spo_album.artists])
+                    self.__song_metadata_dict['nb_tracks'] = getattr(spo_album, 'total_tracks', 0)
+                    # Total discs if available
+                    if hasattr(spo_album, 'total_discs') and getattr(spo_album, 'total_discs'):
+                        self.__song_metadata_dict['nb_discs'] = getattr(spo_album, 'total_discs')
+                    # Year from release date
+                    from deezspot.libutils.metadata_converter import _format_release_date, _get_best_image_url
+                    self.__song_metadata_dict['year'] = _format_release_date(getattr(spo_album, 'release_date', None), 'spotify')
+                    # IDs
+                    if getattr(spo_album, 'ids', None):
+                        self.__song_metadata_dict['upc'] = getattr(spo_album.ids, 'upc', None)
+                        self.__song_metadata_dict['album_id'] = getattr(spo_album.ids, 'spotify', None)
+                    # Prefer Spotify album image
+                    if getattr(spo_album, 'images', None):
+                        img_url = _get_best_image_url(spo_album.images, 'spotify')
+                        if img_url:
+                            self.__song_metadata_dict['image'] = img_url
+            except Exception:
+                pass
 
         self.__c_quality = qualities[self.__quality_download]
         self.__fallback_ids = self.__ids
@@ -280,7 +314,7 @@ class EASY_DW:
         current_track_val = None
         total_tracks_val = None
 
-        if self.__parent == "playlist" and hasattr(self.__preferences, "json_data"):
+        if self.__parent == "playlist" and hasattr(self.__preferences, "json_data") and self.__preferences.json_data:
             playlist_data = self.__preferences.json_data
             
             if isinstance(playlist_data, dict):
@@ -304,7 +338,7 @@ class EASY_DW:
             total_tracks_val = getattr(self.__preferences, 'total_tracks', 0)
             current_track_val = getattr(self.__preferences, 'track_number', 0)
 
-        elif self.__parent == "album" and hasattr(self.__preferences, "json_data"):
+        elif self.__parent == "album" and hasattr(self.__preferences, "json_data") and self.__preferences.json_data:
             album_data = self.__preferences.json_data
             album_id = album_data.ids.deezer
             parent_obj = albumCbObject(
@@ -324,7 +358,8 @@ class EASY_DW:
         It intelligently finds the album information based on the download context.
         """
         # Use the unified metadata converter
-        metadata_dict = track_object_to_dict(track_obj, source_type='deezer')
+        artist_separator = getattr(self.__preferences, 'artist_separator', '; ')
+        metadata_dict = track_object_to_dict(track_obj, source_type='deezer', artist_separator=artist_separator)
         
         # Check for track_position and disk_number in the original API data
         # These might be directly available in the infos_dw dictionary for Deezer tracks
@@ -345,8 +380,25 @@ class EASY_DW:
         custom_dir_format = getattr(self.__preferences, 'custom_dir_format', None)
         custom_track_format = getattr(self.__preferences, 'custom_track_format', None)
         pad_tracks = getattr(self.__preferences, 'pad_tracks', True)
+        self.__song_metadata_dict['artist_separator'] = getattr(self.__preferences, 'artist_separator', '; ')
+        # Inject playlist placeholders if in playlist context
+        try:
+            if self.__parent == 'playlist' and hasattr(self.__preferences, 'json_data') and self.__preferences.json_data:
+                playlist_data = self.__preferences.json_data
+                # Support both Deezer playlistObject and raw dict (Spotify metadata case)
+                if isinstance(playlist_data, dict):
+                    playlist_name = playlist_data.get('name') or playlist_data.get('title') or 'unknown'
+                else:
+                    # playlistObject has 'title'
+                    playlist_name = getattr(playlist_data, 'title', 'unknown')
+                self.__song_metadata_dict['playlist'] = playlist_name
+                # 1-based index already stored in preferences during iteration
+                self.__song_metadata_dict['playlistnum'] = getattr(self.__preferences, 'track_number', None) or 0
+        except Exception:
+            # Non-fatal if playlist metadata is not available
+            pass
         self.__song_path = set_path(
-            self.__song_metadata,
+            self.__song_metadata_dict,
             self.__output_dir,
             self.__song_quality,
             self.__file_format,
@@ -360,7 +412,7 @@ class EASY_DW:
         custom_track_format = getattr(self.__preferences, 'custom_track_format', None)
         pad_tracks = getattr(self.__preferences, 'pad_tracks', True)
         self.__song_path = set_path(
-            self.__song_metadata,
+            self.__song_metadata_dict,
             self.__output_dir,
             self.__song_quality,
             self.__file_format,
@@ -395,11 +447,18 @@ class EASY_DW:
 
     def easy_dw(self) -> Track:
         # Get image URL and enhance metadata
+        pic = None
         if self.__infos_dw.get('__TYPE__') == 'episode':
             pic = self.__infos_dw.get('EPISODE_IMAGE_MD5', '')
+            image = API.choose_img(pic)
         else:
-            pic = self.__infos_dw['ALB_PICTURE']
-        image = API.choose_img(pic)
+            # If using Spotify metadata, prefer the best Spotify image URL from the track object
+            if getattr(self.__preferences, 'spotify_metadata', False) and hasattr(self.__track_obj, 'album') and getattr(self.__track_obj.album, 'images', None):
+                from deezspot.libutils.metadata_converter import _get_best_image_url
+                image = _get_best_image_url(self.__track_obj.album.images, 'spotify')
+            else:
+                pic = self.__infos_dw['ALB_PICTURE']
+                image = API.choose_img(pic)
         self.__song_metadata['image'] = image
         
         # Process image data using unified utility
@@ -472,13 +531,26 @@ class EASY_DW:
                 if self.__c_track.success :
                     parent_obj, current_track_val, total_tracks_val = self._get_parent_context()
                     
-                    done_status = doneObject(ids=self.__track_obj.ids, convert_to=self.__convert_to)
+                    # Compute final path and quality label for Deezer
+                    final_path_val = getattr(self.__c_track, 'song_path', None)
+                    # Deezer quality is directly the selected key or final file format
+                    dz_quality_key = self.__quality_download
+                    download_quality_val = dz_quality_key if dz_quality_key else (self.__c_track.file_format.upper().lstrip('.') if getattr(self.__c_track, 'file_format', None) else None)
+
+                    done_status = doneObject(
+                        ids=self.__track_obj.ids,
+                        convert_to=self.__convert_to,
+                        final_path=final_path_val,
+                        download_quality=download_quality_val
+                    )
                     
                     if self.__parent is None:
                         summary = summaryObject(
                             successful_tracks=[self.__track_obj],
-                            total_successful=1
+                            total_successful=1,
                         )
+                        summary.final_path = final_path_val
+                        summary.download_quality = download_quality_val
                         done_status.summary = summary
                         
                     callback_obj = trackCallbackObject(
@@ -549,21 +621,30 @@ class EASY_DW:
 
                 # If we reach here, the item should be successful and not skipped.
         if current_item.success:
-            if self.__infos_dw.get('__TYPE__') != 'episode': # Assuming pic is for tracks
+            if self.__infos_dw.get('__TYPE__') != 'episode' and pic: # Assuming pic is for tracks
                 current_item.md5_image = pic # Set md5_image for tracks
-            # Apply tags using unified utility with Deezer enhancements
+            # Apply tags using unified utility with Deezer or Spotify enhancements
             from deezspot.deezloader.deegw_api import API_GW
-            enhanced_metadata = add_deezer_enhanced_metadata(
-                self.__song_metadata,
-                self.__infos_dw,
-                self.__ids,
-                API_GW
-            )
-            process_and_tag_track(
-                track=current_item,
-                metadata_dict=enhanced_metadata,
-                source_type='deezer'
-            )
+            use_spotify = getattr(self.__preferences, 'spotify_metadata', False)
+            if use_spotify:
+                enhanced_metadata = add_spotify_enhanced_metadata(self.__song_metadata, self.__track_obj)
+                process_and_tag_track(
+                    track=current_item,
+                    metadata_dict=enhanced_metadata,
+                    source_type='spotify'
+                )
+            else:
+                enhanced_metadata = add_deezer_enhanced_metadata(
+                    self.__song_metadata,
+                    self.__infos_dw,
+                    self.__ids,
+                    API_GW
+                )
+                process_and_tag_track(
+                    track=current_item,
+                    metadata_dict=enhanced_metadata,
+                    source_type='deezer'
+                )
         
         return current_item
 
@@ -580,7 +661,9 @@ class EASY_DW:
             if filesize == 0:
                 song = self.__song_metadata['music']
                 artist = self.__song_metadata['artist']
-                # Switch quality settings to MP3_320.
+                if not self.__recursive_quality:
+                    raise QualityNotFound(f"FLAC not available for {song} - {artist} and recursive quality search is disabled.")
+                # Fallback to MP3_320 if recursive_quality is enabled
                 self.__quality_download = 'MP3_320'
                 self.__file_format = '.mp3'
                 self.__song_path = self.__song_path.rsplit('.', 1)[0] + '.mp3'
@@ -593,51 +676,97 @@ class EASY_DW:
         # Continue with the normal download process.
         try:
             media_list = self.__infos_dw['media_url']['media']
-            song_link = media_list[0]['sources'][0]['url']
 
-            try:
-                crypted_audio = API_GW.song_exist(song_link)
-            except TrackNotFound:
+            # Try all sources for the requested quality before attempting any fallback
+            crypted_audio = None
+            last_error = None
+            for media_entry in media_list:
+                sources = media_entry.get('sources') or []
+                for src in sources:
+                    song_link = src.get('url')
+                    if not song_link:
+                        continue
+                    try:
+                        crypted_audio = API_GW.song_exist(song_link)
+                        if crypted_audio:
+                            last_error = None
+                            break
+                    except Exception as e_try_src:
+                        last_error = e_try_src
+                        continue
+                if crypted_audio:
+                    break
+
+            if not crypted_audio:
                 song = self.__song_metadata['music']
                 artist = self.__song_metadata['artist']
 
                 if self.__file_format == '.flac':
+                    if not self.__recursive_quality:
+                        raise QualityNotFound(f"FLAC not available for {song} - {artist} and recursive quality search is disabled.")
                     logger.warning(f"\n⚠ {song} - {artist} is not available in FLAC format. Trying MP3...")
                     self.__quality_download = 'MP3_320'
                     self.__file_format = '.mp3'
                     self.__song_path = self.__song_path.rsplit('.', 1)[0] + '.mp3'
 
-                    media = Download_JOB.check_sources(
-                        [self.__infos_dw], 'MP3_320'
-                    )
+                    media = Download_JOB.check_sources([self.__infos_dw], 'MP3_320')
                     if media:
                         self.__infos_dw['media_url'] = media[0]
-                        song_link = media[0]['media'][0]['sources'][0]['url']
-                        crypted_audio = API_GW.song_exist(song_link)
+                        media_list = self.__infos_dw['media_url']['media']
+                        # Try all sources for fallback quality
+                        for media_entry in media_list:
+                            sources = media_entry.get('sources') or []
+                            for src in sources:
+                                song_link = src.get('url')
+                                if not song_link:
+                                    continue
+                                try:
+                                    crypted_audio = API_GW.song_exist(song_link)
+                                    if crypted_audio:
+                                        last_error = None
+                                        break
+                                except Exception as e_try_src2:
+                                    last_error = e_try_src2
+                                    continue
+                            if crypted_audio:
+                                break
+                        if not crypted_audio:
+                            raise TrackNotFound(f"Track {song} - {artist} not available in MP3 after FLAC attempt failed (all sources unreachable). Last error: {last_error}")
                     else:
                         raise TrackNotFound(f"Track {song} - {artist} not available in MP3 after FLAC attempt failed (media not found for MP3).")
                 else:
                     if not self.__recursive_quality:
-                        # msg was not defined, provide a more specific message
                         raise QualityNotFound(f"Quality {self.__quality_download} not found for {song} - {artist} and recursive quality search is disabled.")
                     for c_quality in qualities:
                         if self.__quality_download == c_quality:
                             continue
-                        media = Download_JOB.check_sources(
-                            [self.__infos_dw], c_quality
-                        )
+                        media = Download_JOB.check_sources([self.__infos_dw], c_quality)
                         if media:
                             self.__infos_dw['media_url'] = media[0]
-                            song_link = media[0]['media'][0]['sources'][0]['url']
-                            try:
-                                crypted_audio = API_GW.song_exist(song_link)
-                                self.__c_quality = qualities[c_quality]
-                                self.__set_quality()
-                                break
-                            except TrackNotFound:
-                                if c_quality == "MP3_128":
-                                    raise TrackNotFound(f"Error with {song} - {artist}. All available qualities failed, last attempt was {c_quality}. Link: {self.__link}")
-                                continue
+                            media_list = self.__infos_dw['media_url']['media']
+                            # Try all sources for alternative quality
+                            for media_entry in media_list:
+                                sources = media_entry.get('sources') or []
+                                for src in sources:
+                                    song_link = src.get('url')
+                                    if not song_link:
+                                        continue
+                                    try:
+                                        crypted_audio = API_GW.song_exist(song_link)
+                                        if crypted_audio:
+                                            self.__c_quality = qualities[c_quality]
+                                            self.__set_quality()
+                                            last_error = None
+                                            break
+                                    except Exception as e_try_src3:
+                                        last_error = e_try_src3
+                                        continue
+                                if crypted_audio:
+                                    break
+                        if crypted_audio:
+                            break
+                    if not crypted_audio:
+                        raise TrackNotFound(f"Error with {song} - {artist}. All available qualities failed. Last error: {last_error}. Link: {self.__link}")
 
             c_crypted_audio = crypted_audio.iter_content(2048)
             
@@ -697,20 +826,31 @@ class EASY_DW:
 
                 # Add Deezer-specific enhanced metadata and apply tags
                 from deezspot.deezloader.deegw_api import API_GW
-                enhanced_metadata = add_deezer_enhanced_metadata(
-                    self.__song_metadata,
-                    self.__infos_dw,
-                    self.__ids,
-                    API_GW
-                )
-                
-                # Apply tags using unified utility
-                process_and_tag_track(
-                    track=self.__c_track,
-                    metadata_dict=enhanced_metadata,
-                    source_type='deezer',
-                    save_cover=getattr(self.__preferences, 'save_cover', False)
-                )
+                # Build metadata: if using Spotify metadata, enhance for Spotify; else Deezer
+                use_spotify = getattr(self.__preferences, 'spotify_metadata', False)
+                if use_spotify:
+                    enhanced_metadata = add_spotify_enhanced_metadata(self.__song_metadata, self.__track_obj)
+                    process_and_tag_track(
+                        track=self.__c_track,
+                        metadata_dict=enhanced_metadata,
+                        source_type='spotify',
+                        save_cover=getattr(self.__preferences, 'save_cover', False)
+                    )
+                else:
+                    enhanced_metadata = add_deezer_enhanced_metadata(
+                        self.__song_metadata,
+                        self.__infos_dw,
+                        self.__ids,
+                        API_GW
+                    )
+                    
+                    # Apply tags using unified utility
+                    process_and_tag_track(
+                        track=self.__c_track,
+                        metadata_dict=enhanced_metadata,
+                        source_type='deezer',
+                        save_cover=getattr(self.__preferences, 'save_cover', False)
+                    )
 
                 if self.__convert_to:
                     format_name, bitrate = self._parse_format_string(self.__convert_to)
@@ -734,19 +874,28 @@ class EASY_DW:
                             logger.error(f"Audio conversion error: {str(conv_error)}. Proceeding with original format.")
                             register_active_download(path_before_conversion)
 
-                # Apply tags using unified utility with Deezer enhancements
+                # Apply tags using unified utility with Deezer or Spotify enhancements
                 from deezspot.deezloader.deegw_api import API_GW
-                enhanced_metadata = add_deezer_enhanced_metadata(
-                    self.__song_metadata,
-                    self.__infos_dw,
-                    self.__ids,
-                    API_GW
-                )
-                process_and_tag_track(
-                    track=self.__c_track,
-                    metadata_dict=enhanced_metadata,
-                    source_type='deezer'
-                )
+                use_spotify = getattr(self.__preferences, 'spotify_metadata', False)
+                if use_spotify:
+                    enhanced_metadata = add_spotify_enhanced_metadata(self.__song_metadata, self.__track_obj)
+                    process_and_tag_track(
+                        track=self.__c_track,
+                        metadata_dict=enhanced_metadata,
+                        source_type='spotify'
+                    )
+                else:
+                    enhanced_metadata = add_deezer_enhanced_metadata(
+                        self.__song_metadata,
+                        self.__infos_dw,
+                        self.__ids,
+                        API_GW
+                    )
+                    process_and_tag_track(
+                        track=self.__c_track,
+                        metadata_dict=enhanced_metadata,
+                        source_type='deezer'
+                    )
                 self.__c_track.success = True
                 unregister_active_download(self.__song_path)
 
@@ -953,7 +1102,8 @@ class DW_ALBUM:
     def _album_object_to_dict(self, album_obj: albumCbObject) -> dict:
         """Converts an albumObject to a dictionary for tagging and path generation."""
         # Use the unified metadata converter
-        return album_object_to_dict(album_obj, source_type='deezer')
+        artist_separator = getattr(self.__preferences, 'artist_separator', '; ')
+        return album_object_to_dict(album_obj, source_type='deezer', artist_separator=artist_separator)
 
     def _track_object_to_dict(self, track_obj: any, album_obj: albumCbObject) -> dict:
         """Converts a track object to a dictionary with album context."""
@@ -973,10 +1123,12 @@ class DW_ALBUM:
                 genres=getattr(track_obj, 'genres', [])
             )
             # Use the unified metadata converter
-            return track_object_to_dict(full_track, source_type='deezer')
+            artist_separator = getattr(self.__preferences, 'artist_separator', '; ')
+            return track_object_to_dict(full_track, source_type='deezer', artist_separator=artist_separator)
         else:
             # Use the unified metadata converter
-            return track_object_to_dict(track_obj, source_type='deezer')
+            artist_separator = getattr(self.__preferences, 'artist_separator', '; ')
+            return track_object_to_dict(track_obj, source_type='deezer', artist_separator=artist_separator)
 
     def __init__(
         self,
@@ -992,6 +1144,10 @@ class DW_ALBUM:
         self.__recursive_quality = self.__preferences.recursive_quality
         album_obj: albumCbObject = self.__preferences.song_metadata
         self.__song_metadata = self._album_object_to_dict(album_obj)
+        self.__song_metadata['artist_separator'] = getattr(self.__preferences, 'artist_separator', '; ')
+        # New: Spotify metadata context for album-level tagging
+        self.__use_spotify = getattr(self.__preferences, 'spotify_metadata', False)
+        self.__spotify_album_obj = getattr(self.__preferences, 'spotify_album_obj', None)
 
     def dw(self) -> Album:
         from deezspot.deezloader.deegw_api import API_GW
@@ -1004,19 +1160,40 @@ class DW_ALBUM:
         md5_image = infos_dw[0]['ALB_PICTURE']
         image_bytes = API.choose_img(md5_image, size="1400x1400")
         
-        # Convert album object to dictionary for legacy functions
-        album_dict = self._album_object_to_dict(album_obj)
-        album_dict['image'] = image_bytes
-
+        # Choose album_dict source: Spotify if requested and available, else Deezer
+        artist_separator = getattr(self.__preferences, 'artist_separator', '; ')
+        if self.__use_spotify and self.__spotify_album_obj:
+            album_dict = album_object_to_dict(self.__spotify_album_obj, source_type='spotify', artist_separator=artist_separator)
+        else:
+            album_dict = self._album_object_to_dict(album_obj)
+        # Prefer Spotify image if requested and available
+        if self.__use_spotify and self.__spotify_album_obj and getattr(self.__spotify_album_obj, 'images', None):
+            from deezspot.libutils.metadata_converter import _get_best_image_url
+            spo_img_url = _get_best_image_url(self.__spotify_album_obj.images, 'spotify')
+            if spo_img_url:
+                album_dict['image'] = spo_img_url
+            else:
+                album_dict['image'] = image_bytes
+        else:
+            album_dict['image'] = image_bytes
+        
         album = Album(self.__ids)
-        album.image = image_bytes
+        # album.image should be raw bytes; fetch URL if needed
+        if isinstance(album_dict['image'], bytes):
+            album.image = album_dict['image']
+        else:
+            try:
+                from deezspot.libutils.taggers import fetch_and_process_image
+                album.image = fetch_and_process_image(album_dict['image']) or API.choose_img(md5_image, size="1400x1400")
+            except Exception:
+                album.image = API.choose_img(md5_image, size="1400x1400")
         album.md5_image = md5_image
         album.nb_tracks = album_obj.total_tracks
         album.album_name = album_obj.title
         album.upc = album_obj.ids.upc
         album.tags = album_dict
         tracks = album.tracks
-
+        
         medias = Download_JOB.check_sources(infos_dw, self.__quality_download)
         
         album_base_directory = get_album_directory(
@@ -1031,6 +1208,20 @@ class DW_ALBUM:
             save_cover_image(album.image, album_base_directory, "cover.jpg")
             
         total_tracks = len(infos_dw)
+        # If spotify album metadata is requested and available, build a map of spotify track objects by ISRC or index
+        spotify_tracks_by_isrc = {}
+        spotify_tracks_in_order = []
+        if self.__use_spotify and self.__spotify_album_obj and getattr(self.__spotify_album_obj, 'tracks', None):
+            # The spotify album object contains trackAlbumObjects with ids (including isrc)
+            for spo_track in self.__spotify_album_obj.tracks:
+                spotify_tracks_in_order.append(spo_track)
+                try:
+                    isrc_val = getattr(spo_track.ids, 'isrc', None)
+                    if isrc_val:
+                        spotify_tracks_by_isrc[isrc_val.upper()] = spo_track
+                except Exception:
+                    pass
+        
         for a, album_track_obj in enumerate(album_obj.tracks):
             c_infos_dw_item = infos_dw[a] 
             
@@ -1069,6 +1260,25 @@ class DW_ALBUM:
                 c_preferences.track_number = a + 1  # For progress reporting only
                 c_preferences.total_tracks = total_tracks
                 c_preferences.link = f"https://deezer.com/track/{c_preferences.ids}"
+                # Inject Spotify per-track object if available without extra API calls
+                if self.__use_spotify and spotify_tracks_in_order:
+                    spo_track_for_tag = None
+                    # Prefer ISRC matching if possible
+                    deezer_isrc = None
+                    try:
+                        deezer_isrc = c_infos_dw_item.get('ISRC') or c_infos_dw_item.get('isrc')
+                        deezer_isrc = deezer_isrc.upper() if isinstance(deezer_isrc, str) else None
+                    except Exception:
+                        deezer_isrc = None
+                    if deezer_isrc and deezer_isrc in spotify_tracks_by_isrc:
+                        spo_track_for_tag = spotify_tracks_by_isrc[deezer_isrc]
+                    else:
+                        # Fallback to index position
+                        if a < len(spotify_tracks_in_order):
+                            spo_track_for_tag = spotify_tracks_in_order[a]
+                    if spo_track_for_tag:
+                        c_preferences.spotify_metadata = True
+                        c_preferences.spotify_track_obj = spo_track_for_tag
                 
                 current_track_object = EASY_DW(c_infos_dw_item, c_preferences, parent='album').easy_dw()
             except Exception as e:
@@ -1139,7 +1349,8 @@ class DW_PLAYLIST:
 
     def _track_object_to_dict(self, track_obj: any) -> dict:
         # Use the unified metadata converter
-        return track_object_to_dict(track_obj, source_type='deezer')
+        artist_separator = getattr(self.__preferences, 'artist_separator', '; ')
+        return track_object_to_dict(track_obj, source_type='deezer', artist_separator=artist_separator)
 
     def dw(self) -> Playlist:
         playlist_obj: playlistCbObject = self.__preferences.json_data
@@ -1223,13 +1434,13 @@ class DW_PLAYLIST:
             if current_track_object:
                 tracks.append(current_track_object)
                 if current_track_object.success and hasattr(current_track_object, 'song_path') and current_track_object.song_path:
-                    append_track_to_m3u(m3u_path, current_track_object.song_path)
+                    append_track_to_m3u(m3u_path, current_track_object)
 
         if self.__make_zip:
             zip_name = f"{self.__output_dir}/{playlist_obj.title} [playlist {self.__ids}]"
             create_zip(tracks, zip_name=zip_name)
             playlist.zip_path = zip_name
-
+ 
         summary_obj = summaryObject(
             successful_tracks=successful_tracks_cb,
             skipped_tracks=skipped_tracks_cb,
@@ -1239,6 +1450,9 @@ class DW_PLAYLIST:
             total_failed=len(failed_tracks_cb)
         )
         
+        # Attach m3u path to summary
+        summary_obj.m3u_path = m3u_path
+        
         status_obj_done = doneObject(ids=playlist_obj.ids, summary=summary_obj)
         callback_obj_done = playlistCallbackObject(playlist=playlist_obj, status_info=status_obj_done)
         report_progress(
