@@ -2,6 +2,7 @@
 import os
 import json
 import logging
+import re
 from deezspot.deezloader.dee_api import API
 from deezspot.easy_spoty import Spo
 from deezspot.deezloader.deegw_api import API_GW
@@ -62,7 +63,19 @@ from deezspot.models.callback.album import albumObject as albumCbObject
 from deezspot.models.callback.playlist import playlistObject as playlistCbObject
 from deezspot.models.callback.common import IDs
 from deezspot.models.callback.user import userObject
+from rapidfuzz import fuzz
 
+def _sim(a: str, b: str) -> float:
+    a = (a or '').strip().lower()
+    b = (b or '').strip().lower()
+    if not a or not b:
+        return 0.0
+    return fuzz.partial_ratio(a, b) / 100
+
+# Clean for searching on Deezer
+def _remove_parentheses(string: str) -> str:
+    # remove () and [] and {}, as well as anything inside
+    return re.sub(r'\{[^)]*\}', '', re.sub(r'\[[^)]*\]', '', re.sub(r'\([^)]*\)', '', string)))
 
 API()
 
@@ -124,7 +137,9 @@ class DeeLogin:
         bitrate=None,
         save_cover=stock_save_cover,
         market=stock_market,
-        playlist_context=None
+        playlist_context=None,
+        artist_separator: str = "; ",
+        spotify_metadata: bool = False
     ) -> Track:
 
         link_is_valid(link_track)
@@ -145,7 +160,7 @@ class DeeLogin:
             report_progress(reporter=self.progress_reporter, callback_obj=callback_obj)
 
         try:
-            # Get standardized track object using our enhanced API module
+            # Default: Get standardized Deezer track object for tagging
             track_obj = API.get_track(ids)
         except (NoDataApi, MarketAvailabilityError) as e:
             # Try to get fallback track information
@@ -171,10 +186,16 @@ class DeeLogin:
             report_error(e, ids, link_track)
             raise e
 
+        # If requested and provided via context, override with Spotify metadata for tagging
+        if spotify_metadata and playlist_context and playlist_context.get('spotify_track_obj'):
+            track_obj_for_tagging = playlist_context.get('spotify_track_obj')
+        else:
+            track_obj_for_tagging = track_obj
+
         # Set up download preferences
         preferences = Preferences()
         preferences.link = link_track
-        preferences.song_metadata = track_obj  # Use our standardized track object
+        preferences.song_metadata = track_obj_for_tagging  # Use selected track object (Spotify or Deezer) for tagging
         preferences.quality_download = quality_download
         preferences.output_dir = output_dir
         preferences.ids = ids
@@ -191,15 +212,18 @@ class DeeLogin:
         preferences.bitrate = bitrate
         preferences.save_cover = save_cover
         preferences.market = market
+        preferences.artist_separator = artist_separator
+        preferences.spotify_metadata = bool(spotify_metadata)
+        preferences.spotify_track_obj = playlist_context.get('spotify_track_obj') if (playlist_context and playlist_context.get('spotify_track_obj')) else None
 
         if playlist_context:
-            preferences.json_data = playlist_context['json_data']
-            preferences.track_number = playlist_context['track_number']
-            preferences.total_tracks = playlist_context['total_tracks']
-            preferences.spotify_url = playlist_context['spotify_url']
+            preferences.json_data = playlist_context.get('json_data')
+            preferences.track_number = playlist_context.get('track_number')
+            preferences.total_tracks = playlist_context.get('total_tracks')
+            preferences.spotify_url = playlist_context.get('spotify_url')
 
         try:
-            parent = 'playlist' if playlist_context else None
+            parent = 'playlist' if (playlist_context and playlist_context.get('json_data')) else None
             track = DW_TRACK(preferences, parent=parent).dw()
             return track
         except Exception as e:
@@ -225,7 +249,10 @@ class DeeLogin:
         bitrate=None,
         save_cover=stock_save_cover,
         market=stock_market,
-        playlist_context=None
+        playlist_context=None,
+        artist_separator: str = "; ",
+        spotify_metadata: bool = False,
+        spotify_album_obj=None
     ) -> Album:
 
         link_is_valid(link_album)
@@ -272,6 +299,9 @@ class DeeLogin:
         preferences.bitrate = bitrate
         preferences.save_cover = save_cover
         preferences.market = market
+        preferences.artist_separator = artist_separator
+        preferences.spotify_metadata = bool(spotify_metadata)
+        preferences.spotify_album_obj = spotify_album_obj
 
         if playlist_context:
             preferences.json_data = playlist_context['json_data']
@@ -304,7 +334,8 @@ class DeeLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market=stock_market
+        market=stock_market,
+        artist_separator: str = "; "
     ) -> Playlist:
 
         link_is_valid(link_playlist)
@@ -338,6 +369,7 @@ class DeeLogin:
         preferences.bitrate = bitrate
         preferences.save_cover = save_cover
         preferences.market = market
+        preferences.artist_separator = artist_separator
 
         playlist = DW_PLAYLIST(preferences).dw()
 
@@ -386,21 +418,71 @@ class DeeLogin:
     def convert_spoty_to_dee_link_track(self, link_track):
         link_is_valid(link_track)
         ids = get_ids(link_track)
-
+        
+        # Attempt via ISRC first
         track_json = Spo.get_track(ids)
-        external_ids = track_json.get('external_ids')
+        if not track_json:
+            raise TrackNotFound(url=link_track, message="Spotify track metadata fetch failed.")
+        external_ids = track_json.get('external_ids') or {}
+        spo_isrc = (external_ids.get('isrc') or '').upper()
+        spo_title = track_json.get('name', '')
+        spo_album_title = (track_json.get('album') or {}).get('name', '')
+        spo_tracknum = int(track_json.get('track_number') or 0)
+        spo_artists = track_json.get('artists') or []
+        spo_main_artist = (spo_artists[0].get('name') if spo_artists else '') or ''
 
-        if not external_ids or 'isrc' not in external_ids:
-            msg = f"⚠ The track '{track_json.get('name', 'Unknown Track')}' has no ISRC and can't be converted to Deezer link :( ⚠"
-            logger.warning(msg)
-            raise TrackNotFound(url=link_track, message=msg)
-
-        isrc_code = external_ids['isrc']
         try:
-            return self.convert_isrc_to_dee_link_track(isrc_code)
-        except TrackNotFound as e:
-            logger.error(f"Failed to convert Spotify track {link_track} (ISRC: {isrc_code}) to Deezer link: {e.message}")
-            raise TrackNotFound(url=link_track, message=f"Failed to find Deezer equivalent for ISRC {isrc_code} from Spotify track {link_track}: {e.message}") from e
+            dz = API.get_track_json(f"isrc:{spo_isrc}")
+            if dz and dz.get('id'):
+                dz_json = dz
+                tn = (dz_json.get('track_position') or dz_json.get('track_number') or 0)
+                title_match = max(
+                    _sim(_remove_parentheses(spo_title), _remove_parentheses(dz_json.get('title', ''))),
+                    _sim(_remove_parentheses(spo_title), _remove_parentheses(dz_json.get('title_short', '')))
+                )
+                album_match = _sim(spo_album_title, (dz_json.get('album') or {}).get('title', ''))
+                t_isrc = (dz_json.get('isrc') or '').upper()
+                # Enforce ISRC match strictly in ISRC lookup path
+                if (
+                    t_isrc and spo_isrc and t_isrc == spo_isrc and
+                    title_match >= 0.90 and album_match >= 0.90 and tn == spo_tracknum
+                ):
+                    return f"https://www.deezer.com/track/{dz_json.get('id')}"
+        except Exception:
+            pass
+        
+        # Fallback: search by title + artist + album
+        query = f'"track:\'{spo_title}\' artist:\'{spo_main_artist}\' album:\'{spo_album_title}\'"'
+        try:
+            candidates = API.search_tracks_raw(query, limit=5)
+        except Exception:
+            candidates = []
+        
+        for cand in candidates:
+            title_match = max(
+                _sim(_remove_parentheses(spo_title), _remove_parentheses(cand.get('title', ''))),
+                _sim(_remove_parentheses(spo_title), _remove_parentheses(cand.get('title_short', '')))
+            )
+            if title_match < 0.90:
+                continue
+            c_id = cand.get('id')
+            if not c_id:
+                continue
+            try:
+                dzc = API.get_track_json(str(c_id))
+            except Exception:
+                continue
+            # Validate using track number and ISRC to be safe
+            tn = (dzc.get('track_position') or dzc.get('track_number') or 0)
+            if tn != spo_tracknum:
+                continue
+            t_isrc = (dzc.get('isrc') or '').upper()
+            # Enforce ISRC strictly in fallback path as well: require present and equal
+            if not spo_isrc or not t_isrc or t_isrc != spo_isrc:
+                continue
+            return f"https://www.deezer.com/track/{c_id}"
+        
+        raise TrackNotFound(url=link_track, message=f"Failed to find Deezer equivalent for ISRC {spo_isrc} from Spotify track {link_track}")
 
     def convert_isrc_to_dee_link_track(self, isrc_code: str) -> str:
         if not isinstance(isrc_code, str) or not isrc_code:
@@ -428,75 +510,50 @@ class DeeLogin:
     def convert_spoty_to_dee_link_album(self, link_album):
         link_is_valid(link_album)
         ids = get_ids(link_album)
-        link_dee = None
-
+        
         spotify_album_data = Spo.get_album(ids)
-
-        # Method 1: Try UPC
-        try:
-            external_ids = spotify_album_data.get('external_ids')
-            if external_ids and 'upc' in external_ids:
-                upc_base = str(external_ids['upc']).lstrip('0')
-                if upc_base:
-                    logger.debug(f"Attempting Deezer album search with UPC: {upc_base}")
-                    try:
-                        deezer_album_obj = API.get_album(f"upc:{upc_base}")
-                        if deezer_album_obj and deezer_album_obj.type and deezer_album_obj.ids and deezer_album_obj.ids.deezer:
-                            link_dee = f"https://www.deezer.com/{deezer_album_obj.type}/{deezer_album_obj.ids.deezer}"
-                            logger.info(f"Found Deezer album via UPC: {link_dee}")
-                    except NoDataApi:
-                        logger.debug(f"No Deezer album found for UPC: {upc_base}")
-                    except Exception as e_upc_search:
-                        logger.warning(f"Error during Deezer API call for UPC {upc_base}: {e_upc_search}")
-            else:
-                logger.debug("No UPC found in Spotify data for album link conversion.")
-        except Exception as e_upc_block:
-            logger.error(f"Error processing UPC for album {link_album}: {e_upc_block}")
-
-        # Method 2: Try ISRC if UPC failed
-        if not link_dee:
-            logger.debug(f"UPC method failed or skipped for {link_album}. Attempting ISRC method.")
+        if not spotify_album_data:
+            raise AlbumNotFound(f"Failed to fetch Spotify album metadata for {link_album}")
+        
+        spo_album_title = spotify_album_data.get('name', '')
+        spo_artists = spotify_album_data.get('artists') or []
+        spo_main_artist = (spo_artists[0].get('name') if spo_artists else '') or ''
+        external_ids = spotify_album_data.get('external_ids') or {}
+        spo_upc = str(external_ids.get('upc') or '').strip()
+        
+        # Try UPC first
+        if spo_upc:
             try:
-                spotify_total_tracks = spotify_album_data.get('total_tracks')
-                spotify_tracks_items = spotify_album_data.get('tracks', {}).get('items', [])
+                dz_album = API.get_album_json(f"upc:{spo_upc}")
+                if dz_album.get('id') and _sim(spo_album_title, dz_album.get('title', '')) >= 0.90:
+                    return f"https://www.deezer.com/album/{dz_album.get('id')}"
+            except Exception:
+                pass
+        
+        # Fallback: title search
+        q = f'"{spo_album_title}" {spo_main_artist}'.strip()
+        try:
+            candidates = API.search_albums_raw(q, limit=5)
+        except Exception:
+            candidates = []
 
-                if not spotify_tracks_items:
-                    logger.warning(f"No track items in Spotify data for {link_album} to attempt ISRC lookup.")
-                else:
-                    for track_item in spotify_tracks_items:
-                        try:
-                            track_spotify_link = track_item.get('external_urls', {}).get('spotify')
-                            if not track_spotify_link: continue
+        for cand in candidates:
+            if _sim(spo_album_title, cand.get('title', '')) < 0.90:
+                continue
+            c_id = cand.get('id')
+            if not c_id:
+                continue
+            try:
+                dzc = API.get_album_json(str(c_id))
+            except Exception:
+                continue
+            upc = str(dzc.get('upc') or '').strip()
+            if spo_upc and upc and spo_upc != upc:
+                continue
+            link_dee = f"https://www.deezer.com/album/{c_id}"
+            return link_dee
 
-                            spotify_track_info = Spo.get_track(track_spotify_link)
-                            isrc_value = spotify_track_info.get('external_ids', {}).get('isrc')
-                            if not isrc_value: continue
-                            
-                            logger.debug(f"Attempting Deezer track search with ISRC: {isrc_value}")
-                            deezer_track_obj = API.get_track(f"isrc:{isrc_value}")
-
-                            if deezer_track_obj and deezer_track_obj.album and deezer_track_obj.album.ids.deezer:
-                                deezer_album_id = deezer_track_obj.album.ids.deezer
-                                full_deezer_album_obj = API.get_album(deezer_album_id)
-                                if (full_deezer_album_obj and
-                                    full_deezer_album_obj.total_tracks == spotify_total_tracks and
-                                    full_deezer_album_obj.type and full_deezer_album_obj.ids and full_deezer_album_obj.ids.deezer):
-                                    link_dee = f"https://www.deezer.com/{full_deezer_album_obj.type}/{full_deezer_album_obj.ids.deezer}"
-                                    logger.info(f"Found Deezer album via ISRC ({isrc_value}): {link_dee}")
-                                    break
-                        except NoDataApi:
-                            logger.debug(f"No Deezer track/album found for ISRC: {isrc_value}")
-                        except Exception as e_isrc_track_search:
-                            logger.warning(f"Error during Deezer search for ISRC {isrc_value}: {e_isrc_track_search}")
-                    if not link_dee:
-                        logger.warning(f"ISRC method completed for {link_album}, but no matching Deezer album found.")
-            except Exception as e_isrc_block:
-                logger.error(f"Error during ISRC processing block for {link_album}: {e_isrc_block}")
-
-        if not link_dee:
-            raise AlbumNotFound(f"Failed to convert Spotify album link {link_album} to a Deezer link after all attempts.")
-
-        return link_dee
+        raise AlbumNotFound(f"Failed to convert Spotify album link {link_album} to a Deezer link after all attempts.")
 
     def download_trackspo(
         self, link_track,
@@ -515,11 +572,27 @@ class DeeLogin:
         bitrate=None,
         save_cover=stock_save_cover,
         market=stock_market,
-        playlist_context=None
+        playlist_context=None,
+        artist_separator: str = "; ",
+        spotify_metadata: bool = False
     ) -> Track:
 
         link_dee = self.convert_spoty_to_dee_link_track(link_track)
 
+        # If requested, prepare Spotify track object for tagging in preferences via playlist_context
+        if spotify_metadata:
+            try:
+                from deezspot.spotloader.__spo_api__ import tracking as spo_tracking
+                spo_ids = get_ids(link_track)
+                spo_track_obj = spo_tracking(spo_ids)
+                if spo_track_obj:
+                    if playlist_context is None:
+                        playlist_context = {}
+                    playlist_context = dict(playlist_context)
+                    playlist_context['spotify_track_obj'] = spo_track_obj
+            except Exception:
+                pass
+
         track = self.download_trackdee(
             link_dee,
             output_dir=output_dir,
@@ -537,7 +610,9 @@ class DeeLogin:
             bitrate=bitrate,
             save_cover=save_cover,
             market=market,
-            playlist_context=playlist_context
+            playlist_context=playlist_context,
+            artist_separator=artist_separator,
+            spotify_metadata=spotify_metadata
         )
 
         return track
@@ -560,11 +635,25 @@ class DeeLogin:
         bitrate=None,
         save_cover=stock_save_cover,
         market=stock_market,
-        playlist_context=None
+        playlist_context=None,
+        artist_separator: str = "; ",
+        spotify_metadata: bool = False
     ) -> Album:
 
         link_dee = self.convert_spoty_to_dee_link_album(link_album)
 
+        spotify_album_obj = None
+        if spotify_metadata:
+            try:
+                # Fetch full Spotify album with tracks once and convert to albumObject
+                from deezspot.spotloader.__spo_api__ import tracking_album as spo_tracking_album
+                spo_ids = get_ids(link_album)
+                spotify_album_json = Spo.get_album(spo_ids)
+                if spotify_album_json:
+                    spotify_album_obj = spo_tracking_album(spotify_album_json)
+            except Exception:
+                spotify_album_obj = None
+
         album = self.download_albumdee(
             link_dee, output_dir,
             quality_download, recursive_quality,
@@ -580,7 +669,10 @@ class DeeLogin:
             bitrate=bitrate,
             save_cover=save_cover,
             market=market,
-            playlist_context=playlist_context
+            playlist_context=playlist_context,
+            artist_separator=artist_separator,
+            spotify_metadata=spotify_metadata,
+            spotify_album_obj=spotify_album_obj
         )
 
         return album
@@ -602,7 +694,9 @@ class DeeLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market=stock_market
+        market=stock_market,
+        artist_separator: str = "; ",
+        spotify_metadata: bool = False
     ) -> Playlist:
 
         link_is_valid(link_playlist)
@@ -622,6 +716,7 @@ class DeeLogin:
             from deezspot.models.callback.playlist import (
                 artistTrackPlaylistObject, 
                 albumTrackPlaylistObject,
+                artistAlbumTrackPlaylistObject,
                 trackPlaylistObject
             )
             
@@ -643,7 +738,6 @@ class DeeLogin:
             # Process album artists
             album_artists = []
             if album_info.get('artists'):
-                from deezspot.models.callback.playlist import artistAlbumTrackPlaylistObject
                 album_artists = [
                     artistAlbumTrackPlaylistObject(
                         name=artist.get('name'),
@@ -731,12 +825,19 @@ class DeeLogin:
                 continue
 
             try:
-                playlist_context = {
+                playlist_ctx = {
                     'json_data': playlist_json,
                     'track_number': index,
                     'total_tracks': total_tracks,
                     'spotify_url': link_track
                 }
+                # Attach Spotify track object for tagging if requested
+                if spotify_metadata:
+                    try:
+                        from deezspot.spotloader.__spo_api__ import json_to_track_playlist_object
+                        playlist_ctx['spotify_track_obj'] = json_to_track_playlist_object(track_info)
+                    except Exception:
+                        pass
                 downloaded_track = self.download_trackspo(
                     link_track,
                     output_dir=output_dir, quality_download=quality_download,
@@ -745,34 +846,27 @@ class DeeLogin:
                     custom_track_format=custom_track_format, pad_tracks=pad_tracks,
                     initial_retry_delay=initial_retry_delay, retry_delay_increase=retry_delay_increase,
                     max_retries=max_retries, convert_to=convert_to, bitrate=bitrate,
-                    save_cover=save_cover, market=market, playlist_context=playlist_context
+                    save_cover=save_cover, market=market, playlist_context=playlist_ctx,
+                    artist_separator=artist_separator, spotify_metadata=spotify_metadata
                 )
                 tracks.append(downloaded_track)
                 
                 # After download, check status for summary
-                track_obj_for_cb = trackCbObject(title=track_name, artists=[artistTrackObject(name=artist_name)])
                 if getattr(downloaded_track, 'was_skipped', False):
-                    skipped_tracks_cb.append(track_obj_for_cb)
+                    skipped_tracks_cb.append(playlist_obj.tracks[index-1])
                 elif downloaded_track.success:
-                    successful_tracks_cb.append(track_obj_for_cb)
+                    successful_tracks_cb.append(playlist_obj.tracks[index-1])
                 else:
-                    failed_tracks_cb.append(failedTrackObject(
-                        track=track_obj_for_cb,
-                        reason=getattr(downloaded_track, 'error_message', 'Unknown reason')
-                    ))
+                    failed_tracks_cb.append(failedTrackObject(track=playlist_obj.tracks[index-1], reason=getattr(downloaded_track, 'error_message', 'Unknown reason')))
+            except Exception as e:
+                logger.error(f"Track '{track_name}' in playlist '{playlist_obj.title}' failed: {e}")
+                failed_tracks_cb.append(failedTrackObject(track=playlist_obj.tracks[index-1], reason=str(e)))
+                current_track_object = Track({'music': track_name, 'artist': artist_name}, None, None, None, link_track, None)
+                current_track_object.success = False
+                current_track_object.error_message = str(e)
+                tracks.append(current_track_object)
 
-            except (TrackNotFound, NoDataApi) as e:
-                logger.error(f"Failed to download track: {track_name} - {artist_name}: {e}")
-                failed_track_obj = trackCbObject(title=track_name, artists=[artistTrackObject(name=artist_name)])
-                failed_tracks_cb.append(failedTrackObject(track=failed_track_obj, reason=str(e)))
-                # Create a placeholder for the failed item
-                failed_track = Track(
-                    tags={'name': track_name, 'artist': artist_name},
-                    song_path=None, file_format=None, quality=None, link=link_track, ids=None
-                )
-                failed_track.success = False
-                failed_track.error_message = str(e)
-                tracks.append(failed_track)
+        # Finalize summary and callbacks (existing logic continues below in file)...
 
         total_from_spotify = playlist_json['tracks']['total']
         processed_count = len(successful_tracks_cb) + len(skipped_tracks_cb) + len(failed_tracks_cb)
@@ -783,6 +877,9 @@ class DeeLogin:
                 f"but only {processed_count} were processed. This might indicate that not all pages of tracks were retrieved from Spotify."
             )
 
+        from deezspot.libutils.write_m3u import write_tracks_to_m3u
+        m3u_path = write_tracks_to_m3u(output_dir, playlist_obj.title, tracks)
+
         summary_obj = summaryObject(
             successful_tracks=successful_tracks_cb,
             skipped_tracks=skipped_tracks_cb,
@@ -791,14 +888,12 @@ class DeeLogin:
             total_skipped=len(skipped_tracks_cb),
             total_failed=len(failed_tracks_cb)
         )
-        
+        # Include m3u path in summary and callback
+        summary_obj.m3u_path = m3u_path
         status_obj_done = doneObject(ids=playlist_obj.ids, summary=summary_obj)
         callback_obj_done = playlistCallbackObject(playlist=playlist_obj, status_info=status_obj_done)
         report_progress(reporter=self.progress_reporter, callback_obj=callback_obj_done)
 
-        from deezspot.libutils.write_m3u import write_tracks_to_m3u
-        m3u_path = write_tracks_to_m3u(output_dir, playlist_obj.title, tracks)
-
         if make_zip:
             zip_name = f"{output_dir}/playlist_{sanitize_name(playlist_obj.title)}.zip"
             create_zip(tracks, zip_name=zip_name)
@@ -822,7 +917,8 @@ class DeeLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market=stock_market
+        market=stock_market,
+        artist_separator: str = "; "
     ) -> Track:
 
         query = f"track:{song} artist:{artist}"
@@ -852,7 +948,8 @@ class DeeLogin:
             convert_to=convert_to,
             bitrate=bitrate,
             save_cover=save_cover,
-            market=market
+            market=market,
+            artist_separator=artist_separator
         )
 
         return track
@@ -874,7 +971,8 @@ class DeeLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market=stock_market
+        market=stock_market,
+        artist_separator: str = "; "
     ) -> Episode:
         
         logger.warning("Episode download logic is not fully refactored and might not work as expected with new reporting.")
@@ -914,11 +1012,12 @@ class DeeLogin:
         preferences.save_cover = save_cover
         preferences.is_episode = True
         preferences.market = market
+        preferences.artist_separator = artist_separator
 
         episode = DW_EPISODE(preferences).dw()
 
         return episode
-    
+        
     def download_smart(
         self, link,
         output_dir=stock_output,
@@ -936,7 +1035,8 @@ class DeeLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market=stock_market
+        market=stock_market,
+        artist_separator: str = "; "
     ) -> Smart:
 
         link_is_valid(link)
@@ -964,7 +1064,7 @@ class DeeLogin:
                 custom_track_format=custom_track_format, pad_tracks=pad_tracks,
                 initial_retry_delay=initial_retry_delay, retry_delay_increase=retry_delay_increase,
                 max_retries=max_retries, convert_to=convert_to, bitrate=bitrate,
-                save_cover=save_cover, market=market
+                save_cover=save_cover, market=market, artist_separator=artist_separator
             )
             smart.type = "track"
             smart.track = track
@@ -979,7 +1079,7 @@ class DeeLogin:
                 pad_tracks=pad_tracks, initial_retry_delay=initial_retry_delay,
                 retry_delay_increase=retry_delay_increase, max_retries=max_retries,
                 convert_to=convert_to, bitrate=bitrate, save_cover=save_cover,
-                market=market
+                market=market, artist_separator=artist_separator
             )
             smart.type = "album"
             smart.album = album
@@ -994,7 +1094,7 @@ class DeeLogin:
                 pad_tracks=pad_tracks, initial_retry_delay=initial_retry_delay,
                 retry_delay_increase=retry_delay_increase, max_retries=max_retries,
                 convert_to=convert_to, bitrate=bitrate, save_cover=save_cover,
-                market=market
+                market=market, artist_separator=artist_separator
             )
             smart.type = "playlist"
             smart.playlist = playlist
