@@ -104,7 +104,13 @@ class EASY_DW:
         self.__link = preferences.link
         self.__output_dir = preferences.output_dir
         self.__song_metadata = preferences.song_metadata
-        self.__song_metadata_dict = _track_object_to_dict(self.__song_metadata)
+        # Convert song metadata to dict with configured artist separator
+        artist_separator = getattr(preferences, 'artist_separator', '; ')
+        if parent == 'album' and hasattr(self.__song_metadata, 'album'):
+            # When iterating album tracks later we will still need the separator, but for initial dict, use track conversion
+            self.__song_metadata_dict = track_object_to_dict(self.__song_metadata, source_type='spotify', artist_separator=artist_separator)
+        else:
+            self.__song_metadata_dict = track_object_to_dict(self.__song_metadata, source_type='spotify', artist_separator=artist_separator)
         self.__not_interface = preferences.not_interface
         self.__quality_download = preferences.quality_download or "NORMAL"
         self.__recursive_download = preferences.recursive_download
@@ -136,6 +142,22 @@ class EASY_DW:
         custom_dir_format = getattr(self.__preferences, 'custom_dir_format', None)
         custom_track_format = getattr(self.__preferences, 'custom_track_format', None)
         pad_tracks = getattr(self.__preferences, 'pad_tracks', True)
+        # Ensure the separator is available to formatting utils for indexed placeholders
+        self.__song_metadata_dict['artist_separator'] = getattr(self.__preferences, 'artist_separator', '; ')
+        # Inject playlist placeholders if in playlist context
+        try:
+            if self.__parent == 'playlist' and hasattr(self.__preferences, 'json_data') and self.__preferences.json_data:
+                playlist_data = self.__preferences.json_data
+                playlist_name = None
+                if isinstance(playlist_data, dict):
+                    playlist_name = playlist_data.get('name') or playlist_data.get('title')
+                if not playlist_name and hasattr(playlist_data, 'title'):
+                    playlist_name = getattr(playlist_data, 'title')
+                self.__song_metadata_dict['playlist'] = playlist_name or 'unknown'
+                self.__song_metadata_dict['playlistnum'] = getattr(self.__preferences, 'track_number', None) or 0
+        except Exception:
+            # If playlist info missing, skip silently
+            pass
         self.__song_path = set_path(
             self.__song_metadata_dict,
             self.__output_dir,
@@ -150,6 +172,7 @@ class EASY_DW:
         custom_dir_format = getattr(self.__preferences, 'custom_dir_format', None)
         custom_track_format = getattr(self.__preferences, 'custom_track_format', None)
         pad_tracks = getattr(self.__preferences, 'pad_tracks', True)
+        self.__song_metadata_dict['artist_separator'] = getattr(self.__preferences, 'artist_separator', '; ')
         self.__song_path = set_path(
             self.__song_metadata_dict,
             self.__output_dir,
@@ -200,7 +223,7 @@ class EASY_DW:
             parent_info = {
                 "type": "album",
                 "title": album_meta.title,
-                "artist": "; ".join([a.name for a in album_meta.artists]),
+                "artist": getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in album_meta.artists]),
                 "total_tracks": total_tracks_val,
                 "url": f"https://open.spotify.com/album/{album_meta.ids.spotify if album_meta.ids else ''}"
             }
@@ -313,7 +336,7 @@ class EASY_DW:
 
         except Exception as e:
             song_title = self.__song_metadata.title
-            artist_name = "; ".join([a.name for a in self.__song_metadata.artists])
+            artist_name = getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in self.__song_metadata.artists])
             error_message = f"Download failed for '{song_title}' by '{artist_name}' (URL: {self.__link}). Original error: {str(e)}"
             logger.error(error_message)
             traceback.print_exc()
@@ -334,7 +357,7 @@ class EASY_DW:
         # This handles cases where download_try didn't raise an exception but self.__c_track.success is still False.
         if hasattr(self, '_EASY_DW__c_track') and self.__c_track and not self.__c_track.success:
             song_title = self.__song_metadata.title
-            artist_name = "; ".join([a.name for a in self.__song_metadata.artists])
+            artist_name = getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in self.__song_metadata.artists])
             original_error_msg = getattr(self.__c_track, 'error_message', "Download failed for an unspecified reason after attempt.")
             error_msg_template = "Cannot download '{title}' by '{artist}'. Reason: {reason}"
             final_error_msg = error_msg_template.format(title=song_title, artist=artist_name, reason=original_error_msg)
@@ -362,7 +385,7 @@ class EASY_DW:
     def download_try(self) -> Track:
         current_title = self.__song_metadata.title
         current_album = self.__song_metadata.album.title if self.__song_metadata.album else ''
-        current_artist = "; ".join([a.name for a in self.__song_metadata.artists])
+        current_artist = getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in self.__song_metadata.artists])
 
         # Call the new check_track_exists function from skip_detection.py
         # It needs: original_song_path, title, album, convert_to, logger
@@ -468,7 +491,20 @@ class EASY_DW:
                             # Real-time download path
                             duration = self.__song_metadata_dict["duration"]
                             if duration > 0:
-                                rate_limit = total_size / duration
+                                # Base rate limit in bytes per second to match real-time
+                                base_rate_limit = total_size / duration
+                                # Multiplier handling (0 disables pacing, 1=real-time, up to 10x)
+                                m = getattr(self.__preferences, 'real_time_multiplier', 1)
+                                try:
+                                    m = int(m)
+                                except Exception:
+                                    m = 1
+                                if m < 0:
+                                    m = 0
+                                if m > 10:
+                                    m = 10
+                                pacing_enabled = m > 0
+                                rate_limit = base_rate_limit * m if pacing_enabled else None
                                 chunk_size = 4096
                                 bytes_written = 0
                                 start_time = time.time()
@@ -491,7 +527,7 @@ class EASY_DW:
                                     if current_percentage > self._last_reported_percentage:
                                         self._last_reported_percentage = current_percentage
                                         
-                                                                                                        # Report real-time progress
+                                        # Report real-time progress
                                         report_track_realtime_progress(
                                             track_obj=track_obj,
                                             time_elapsed=int((current_time - start_time) * 1000),
@@ -501,10 +537,12 @@ class EASY_DW:
                                             total_tracks=total_tracks_val
                                         )
                                         
-                                    # Rate limiting (if needed)
-                                    expected_time = bytes_written / rate_limit
-                                    if expected_time > (time.time() - start_time):
-                                        time.sleep(expected_time - (time.time() - start_time))
+                                    # Rate limiting (if pacing is enabled)
+                                    if pacing_enabled and rate_limit:
+                                        expected_time = bytes_written / rate_limit
+                                        elapsed = time.time() - start_time
+                                        if expected_time > elapsed:
+                                            time.sleep(expected_time - elapsed)
                         else:
                             # Non real-time download path
                             data = c_stream.read(total_size)
@@ -571,7 +609,7 @@ class EASY_DW:
                         os.remove(self.__song_path)
                     # Add track info to exception    
                     track_name = self.__song_metadata.title
-                    artist_name = "; ".join([a.name for a in self.__song_metadata.artists])
+                    artist_name = getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in self.__song_metadata.artists])
                     final_error_msg = f"Maximum retry limit reached for '{track_name}' by '{artist_name}' (local: {max_retries}, global: {GLOBAL_MAX_RETRIES}). Last error: {str(e)}"
                     # Store error on track object
                     if hasattr(self, '_EASY_DW__c_track') and self.__c_track:
@@ -665,15 +703,41 @@ class EASY_DW:
                 total_skipped=len(skipped_track_list),
                 total_failed=0
             )
+            # Enrich summary with final path and quality
+            try:
+                final_path_val = getattr(self.__c_track, 'song_path', None)
+            except Exception:
+                final_path_val = None
+            quality_key_single = self.__quality_download
+            sp_quality_map_single = {
+                'NORMAL': 'OGG_96',
+                'HIGH': 'OGG_160',
+                'VERY_HIGH': 'OGG_320'
+            }
+            summary_obj.final_path = final_path_val
+            summary_obj.download_quality = sp_quality_map_single.get(quality_key_single, 'OGG')
 
         # Report track done status
+        # Compute final path and quality label
+        final_path_val = getattr(self.__c_track, 'song_path', None)
+        # Map Spotify quality to OGG bitrate label
+        quality_key = self.__quality_download if hasattr(self, '_EASY_DW__quality_download') else getattr(self, '_EASY_DW__quality_download', None)
+        quality_key = self.__quality_download if quality_key is None else quality_key
+        sp_quality_map = {
+            'NORMAL': 'OGG_96',
+            'HIGH': 'OGG_160',
+            'VERY_HIGH': 'OGG_320'
+        }
+        download_quality_val = sp_quality_map.get(quality_key, 'OGG')
         report_track_done(
             track_obj=track_obj,
             preferences=self.__preferences,
             summary=summary_obj,
             parent_obj=parent_obj,
             current_track=current_track_val,
-            total_tracks=total_tracks_val
+            total_tracks=total_tracks_val,
+            final_path=final_path_val,
+            download_quality=download_quality_val
         )
 
         if hasattr(self, '_EASY_DW__c_track') and self.__c_track and self.__c_track.success:
@@ -736,7 +800,7 @@ class EASY_DW:
                     if os.path.exists(self.__song_path):
                         os.remove(self.__song_path) # Clean up partial file
                     track_name = self.__song_metadata.title
-                    artist_name = "; ".join([a.name for a in self.__song_metadata.artists])
+                    artist_name = getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in self.__song_metadata.artists])
                     final_error_msg = f"Maximum retry limit reached for '{track_name}' by '{artist_name}' (local: {max_retries}, global: {GLOBAL_MAX_RETRIES}). Last error: {str(e)}"
                     if hasattr(self, '_EASY_DW__c_episode') and self.__c_episode:
                         self.__c_episode.success = False
@@ -756,7 +820,20 @@ class EASY_DW:
                 if self.__real_time_dl and self.__song_metadata_dict.get("duration") and self.__song_metadata_dict["duration"] > 0:
                     # Restored Real-time download logic for episodes
                     duration = self.__song_metadata_dict["duration"]
-                    rate_limit = total_size / duration
+                    # Base rate to match real-time
+                    base_rate_limit = total_size / duration if duration > 0 else None
+                    # Multiplier handling
+                    m = getattr(self.__preferences, 'real_time_multiplier', 1)
+                    try:
+                        m = int(m)
+                    except Exception:
+                        m = 1
+                    if m < 0:
+                        m = 0
+                    if m > 10:
+                        m = 10
+                    pacing_enabled = (base_rate_limit is not None) and m > 0
+                    rate_limit = (base_rate_limit * m) if pacing_enabled else None
                     chunk_size = 4096
                     bytes_written = 0
                     start_time = time.time()
@@ -769,10 +846,11 @@ class EASY_DW:
                             bytes_written += len(chunk)
                             # Optional: Real-time progress reporting for episodes (can be added here if desired)
                             # Matching the style of download_try, no specific progress report inside this loop for episodes by default.
-                            expected_time = bytes_written / rate_limit
-                            elapsed_time = time.time() - start_time
-                            if expected_time > elapsed_time:
-                                time.sleep(expected_time - elapsed_time)
+                            if pacing_enabled and rate_limit:
+                                expected_time = bytes_written / rate_limit
+                                elapsed_time = time.time() - start_time
+                                if expected_time > elapsed_time:
+                                    time.sleep(expected_time - elapsed_time)
                     except Exception as e_realtime:
                         # If any error occurs during real-time download, clean up
                         if not c_stream.closed:
@@ -788,7 +866,7 @@ class EASY_DW:
                                 pass
                         unregister_active_download(self.__song_path)
                         episode_title = self.__song_metadata.title
-                        artist_name = "; ".join([a.name for a in self.__song_metadata.artists])
+                        artist_name = getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in self.__song_metadata.artists])
                         final_error_msg = f"Error during real-time download for episode '{episode_title}' by '{artist_name}' (URL: {self.__link}). Error: {str(e_realtime)}"
                         logger.error(final_error_msg)
                         if hasattr(self, '_EASY_DW__c_episode') and self.__c_episode:
@@ -814,7 +892,7 @@ class EASY_DW:
                                 pass
                         unregister_active_download(self.__song_path)
                         episode_title = self.__song_metadata.title
-                        artist_name = "; ".join([a.name for a in self.__song_metadata.artists])
+                        artist_name = getattr(self.__preferences, 'artist_separator', '; ').join([a.name for a in self.__song_metadata.artists])
                         final_error_msg = f"Error during standard download for episode '{episode_title}' by '{artist_name}' (URL: {self.__link}). Error: {str(e_standard)}"
                         logger.error(final_error_msg)
                         if hasattr(self, '_EASY_DW__c_episode') and self.__c_episode:
@@ -953,7 +1031,8 @@ class DW_ALBUM:
         album.upc = album_obj.ids.upc
         tracks = album.tracks
         album.md5_image = self.__ids
-        album.tags = _album_object_to_dict(self.__album_metadata) # For top-level album tags if needed
+        album.tags = album_object_to_dict(self.__album_metadata, source_type='spotify', artist_separator=getattr(self.__preferences, 'artist_separator', '; ')) # For top-level album tags if needed
+        album.tags['artist_separator'] = getattr(self.__preferences, 'artist_separator', '; ')
         
         album_base_directory = get_album_directory(
             album.tags,
@@ -997,11 +1076,11 @@ class DW_ALBUM:
                 logger.warning(f"Track '{song_tags.get('music')}' from album '{album.album_name}' failed to download. Reason: {track.error_message}")
 
             tracks.append(track)
-
+        
         # Save album cover image
         if self.__preferences.save_cover and album.image and album_base_directory:
             save_cover_image(album.image, album_base_directory, "cover.jpg")
-
+        
         if self.__make_zip:
             song_quality = tracks[0].quality if tracks and tracks[0].quality else 'HIGH' # Fallback quality
             zip_name = create_zip(
@@ -1132,9 +1211,9 @@ class DW_PLAYLIST:
             if track:
                 tracks.append(track)
 
-            # --- Append the final track path to the m3u file using a relative path ---
+            # --- Append the final track to the m3u file with extended format ---
             if track and track.success and hasattr(track, 'song_path') and track.song_path:
-                append_track_to_m3u(m3u_path, track.song_path)
+                append_track_to_m3u(m3u_path, track)
             # ---------------------------------------------------------------------
         
         if self.__make_zip:
@@ -1174,7 +1253,8 @@ class DW_PLAYLIST:
             total_failed=len(failed_tracks_cb)
         )
         
-        report_playlist_done(playlist_obj_for_cb, summary_obj)
+        # Include m3u path in summary and callback
+        report_playlist_done(playlist_obj_for_cb, summary_obj, m3u_path=m3u_path)
         
         return playlist
 
