@@ -35,7 +35,8 @@ from deezspot.libutils.others_settings import (
     stock_zip,
     stock_save_cover,
     stock_real_time_dl,
-    stock_market
+    stock_market,
+    stock_real_time_multiplier
 )
 from deezspot.libutils.logging_utils import logger, ProgressReporter, report_progress
 
@@ -88,6 +89,7 @@ class SpoLogin:
         recursive_download=stock_recursive_download,
         not_interface=stock_not_interface,
         real_time_dl=stock_real_time_dl,
+        real_time_multiplier: int = stock_real_time_multiplier,
         custom_dir_format=None,
         custom_track_format=None,
         pad_tracks=True,
@@ -97,7 +99,8 @@ class SpoLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market: list[str] | None = stock_market
+        market: list[str] | None = stock_market,
+        artist_separator: str = "; "
     ) -> Track:
         song_metadata = None
         try:
@@ -108,10 +111,11 @@ class SpoLogin:
             if song_metadata is None:
                 raise Exception(f"Could not retrieve metadata for track {link_track}. It might not be available or an API error occurred.")
 
-            logger.info(f"Starting download for track: {song_metadata.title} - {'; '.join([a.name for a in song_metadata.artists])}")
+            logger.info(f"Starting download for track: {song_metadata.title} - {artist_separator.join([a.name for a in song_metadata.artists])}")
 
             preferences = Preferences()
             preferences.real_time_dl = real_time_dl
+            preferences.real_time_multiplier = int(real_time_multiplier) if real_time_multiplier is not None else 1
             preferences.link = link_track
             preferences.song_metadata = song_metadata
             preferences.quality_download = quality_download
@@ -135,6 +139,7 @@ class SpoLogin:
                 preferences.bitrate = bitrate
             preferences.save_cover = save_cover
             preferences.market = market
+            preferences.artist_separator = artist_separator
 
             track = DW_TRACK(preferences).dw()
 
@@ -170,6 +175,7 @@ class SpoLogin:
         not_interface=stock_not_interface,
         make_zip=stock_zip,
         real_time_dl=stock_real_time_dl,
+        real_time_multiplier: int = stock_real_time_multiplier,
         custom_dir_format=None,
         custom_track_format=None,
         pad_tracks=True,
@@ -179,7 +185,8 @@ class SpoLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market: list[str] | None = stock_market
+        market: list[str] | None = stock_market,
+        artist_separator: str = "; "
     ) -> Album:
         try:
             link_is_valid(link_album)
@@ -192,10 +199,11 @@ class SpoLogin:
             if song_metadata is None:
                 raise Exception(f"Could not process album metadata for {link_album}. It might not be available in the specified market(s) or an API error occurred.")
 
-            logger.info(f"Starting download for album: {song_metadata.title} - {'; '.join([a.name for a in song_metadata.artists])}")
+            logger.info(f"Starting download for album: {song_metadata.title} - {artist_separator.join([a.name for a in song_metadata.artists])}")
 
             preferences = Preferences()
             preferences.real_time_dl = real_time_dl
+            preferences.real_time_multiplier = int(real_time_multiplier) if real_time_multiplier is not None else 1
             preferences.link = link_album
             preferences.song_metadata = song_metadata
             preferences.quality_download = quality_download
@@ -221,6 +229,7 @@ class SpoLogin:
                 preferences.bitrate = bitrate
             preferences.save_cover = save_cover
             preferences.market = market
+            preferences.artist_separator = artist_separator
 
             album = DW_ALBUM(preferences).dw()
 
@@ -242,6 +251,7 @@ class SpoLogin:
         not_interface=stock_not_interface,
         make_zip=stock_zip,
         real_time_dl=stock_real_time_dl,
+        real_time_multiplier: int = stock_real_time_multiplier,
         custom_dir_format=None,
         custom_track_format=None,
         pad_tracks=True,
@@ -251,7 +261,8 @@ class SpoLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market: list[str] | None = stock_market
+        market: list[str] | None = stock_market,
+        artist_separator: str = "; "
     ) -> Playlist:
         try:
             link_is_valid(link_playlist)
@@ -301,6 +312,7 @@ class SpoLogin:
 
             preferences = Preferences()
             preferences.real_time_dl = real_time_dl
+            preferences.real_time_multiplier = int(real_time_multiplier) if real_time_multiplier is not None else 1
             preferences.link = link_playlist
             preferences.song_metadata = song_metadata_list
             preferences.quality_download = quality_download
@@ -327,6 +339,7 @@ class SpoLogin:
                 preferences.bitrate = bitrate
             preferences.save_cover = save_cover
             preferences.market = market
+            preferences.artist_separator = artist_separator
             
             playlist = DW_PLAYLIST(preferences).dw()
 
@@ -347,6 +360,7 @@ class SpoLogin:
         recursive_download=stock_recursive_download,
         not_interface=stock_not_interface,
         real_time_dl=stock_real_time_dl,
+        real_time_multiplier: int = stock_real_time_multiplier,
         custom_dir_format=None,
         custom_track_format=None,
         pad_tracks=True,
@@ -356,7 +370,8 @@ class SpoLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market: list[str] | None = stock_market
+        market: list[str] | None = stock_market,
+        artist_separator: str = "; "
     ) -> Episode:
         try:
             link_is_valid(link_episode)
@@ -373,6 +388,7 @@ class SpoLogin:
 
             preferences = Preferences()
             preferences.real_time_dl = real_time_dl
+            preferences.real_time_multiplier = int(real_time_multiplier) if real_time_multiplier is not None else 1
             preferences.link = link_episode
             preferences.song_metadata = episode_metadata
             preferences.output_dir = output_dir
@@ -396,6 +412,7 @@ class SpoLogin:
                 preferences.bitrate = bitrate
             preferences.save_cover = save_cover
             preferences.market = market
+            preferences.artist_separator = artist_separator
 
             episode = DW_EPISODE(preferences).dw()
 
@@ -419,6 +436,7 @@ class SpoLogin:
         not_interface=stock_not_interface,
         make_zip=stock_zip,
         real_time_dl=stock_real_time_dl,
+        real_time_multiplier: int = stock_real_time_multiplier,
         custom_dir_format=None,
         custom_track_format=None,
         pad_tracks=True,
@@ -428,7 +446,8 @@ class SpoLogin:
         convert_to=None,
         bitrate=None,
         market: list[str] | None = stock_market,
-        save_cover=stock_save_cover
+        save_cover=stock_save_cover,
+        artist_separator: str = "; "
     ):
         """
         Download all albums (or a subset based on album_type and limit) from an artist.
@@ -459,6 +478,7 @@ class SpoLogin:
                     not_interface=not_interface,
                     make_zip=make_zip,
                     real_time_dl=real_time_dl,
+                    real_time_multiplier=real_time_multiplier,
                     custom_dir_format=custom_dir_format,
                     custom_track_format=custom_track_format,
                     pad_tracks=pad_tracks,
@@ -468,7 +488,8 @@ class SpoLogin:
                     convert_to=convert_to,
                     bitrate=bitrate,
                     market=market,
-                    save_cover=save_cover
+                    save_cover=save_cover,
+                    artist_separator=artist_separator
                 )
                 downloaded_albums.append(downloaded_album)
             return downloaded_albums
@@ -486,6 +507,7 @@ class SpoLogin:
         not_interface=stock_not_interface,
         make_zip=stock_zip,
         real_time_dl=stock_real_time_dl,
+        real_time_multiplier: int = stock_real_time_multiplier,
         custom_dir_format=None,
         custom_track_format=None,
         pad_tracks=True,
@@ -495,7 +517,8 @@ class SpoLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market: list[str] | None = stock_market
+        market: list[str] | None = stock_market,
+        artist_separator: str = "; "
     ) -> Smart:
         try:
             link_is_valid(link)
@@ -519,6 +542,7 @@ class SpoLogin:
                     recursive_download=recursive_download,
                     not_interface=not_interface,
                     real_time_dl=real_time_dl,
+                    real_time_multiplier=real_time_multiplier,
                     custom_dir_format=custom_dir_format,
                     custom_track_format=custom_track_format,
                     pad_tracks=pad_tracks,
@@ -528,7 +552,8 @@ class SpoLogin:
                     convert_to=convert_to,
                     bitrate=bitrate,
                     save_cover=save_cover,
-                    market=market
+                    market=market,
+                    artist_separator=artist_separator
                 )
                 smart.type = "track"
                 smart.track = track
@@ -545,6 +570,7 @@ class SpoLogin:
                     not_interface=not_interface,
                     make_zip=make_zip,
                     real_time_dl=real_time_dl,
+                    real_time_multiplier=real_time_multiplier,
                     custom_dir_format=custom_dir_format,
                     custom_track_format=custom_track_format,
                     pad_tracks=pad_tracks,
@@ -554,7 +580,8 @@ class SpoLogin:
                     convert_to=convert_to,
                     bitrate=bitrate,
                     save_cover=save_cover,
-                    market=market
+                    market=market,
+                    artist_separator=artist_separator
                 )
                 smart.type = "album"
                 smart.album = album
@@ -571,6 +598,7 @@ class SpoLogin:
                     not_interface=not_interface,
                     make_zip=make_zip,
                     real_time_dl=real_time_dl,
+                    real_time_multiplier=real_time_multiplier,
                     custom_dir_format=custom_dir_format,
                     custom_track_format=custom_track_format,
                     pad_tracks=pad_tracks,
@@ -580,7 +608,8 @@ class SpoLogin:
                     convert_to=convert_to,
                     bitrate=bitrate,
                     save_cover=save_cover,
-                    market=market
+                    market=market,
+                    artist_separator=artist_separator
                 )
                 smart.type = "playlist"
                 smart.playlist = playlist
@@ -596,6 +625,7 @@ class SpoLogin:
                     recursive_download=recursive_download,
                     not_interface=not_interface,
                     real_time_dl=real_time_dl,
+                    real_time_multiplier=real_time_multiplier,
                     custom_dir_format=custom_dir_format,
                     custom_track_format=custom_track_format,
                     pad_tracks=pad_tracks,
@@ -605,7 +635,8 @@ class SpoLogin:
                     convert_to=convert_to,
                     bitrate=bitrate,
                     save_cover=save_cover,
-                    market=market
+                    market=market,
+                    artist_separator=artist_separator
                 )
                 smart.type = "episode"
                 smart.episode = episode
